@@ -1,9 +1,14 @@
-from commonscripts import elementwiseMultiply,subset
+from commonscripts import elementwiseMultiply,subset,flatten
 from optimization import *
 from config import default_num_breakpoints
 
 from scipy import linspace, polyval, polyder, interp, poly1d
+
+import matplotlib
+#from sys import platform as osname
+#if osname=='darwin': matplotlib.use('macosx') #avoid popups when using matploblib to savefig on MacOSX
 from pylab import plot,show,savefig,xlabel,ylabel
+
 import re
 import logging
 
@@ -17,13 +22,10 @@ class Bid(object):
       :class:`~bidding.PWLmodel` objects).
     :param iden: identifying string for the bidder
     """
-
-    def __init__(self,model,iden):
-        vars(self).update(locals())
-
-        #add time variables
-        self.segmentsActive,self.fractionsBP=self.model.addTimeVars(self.iden)
-    def output(self): return self.model.output(self.fractionsBP)
+    def __init__(self,model,inputvar,iden,statusvar=True):
+        vars(self).update(locals())        
+    def output(self): 
+        return self.model.output(self.variables,self.iden)
     def trueOutput(self,input): return self.model.trueOutput(input)
     def incOutput(self,input):  return self.model.incOutput(input)
     def plotDeriv(self,**kwargs): return self.model.plotDeriv(**kwargs)
@@ -31,18 +33,36 @@ class Bid(object):
         plotted=self.model.plot(P,showPW=showPW)
         if filename is not None: savefig(filename)
         return plotted
-    def constraints(self,inputVar,status):
+    def constraints(self):
         '''Create the constraints for a bid by calling its 
         model.constraint() method.'''
         return self.model.constraints(
-            S=self.segmentsActive,
-            F=self.fractionsBP,
-            inputVar=inputVar,
-            status=status,
-            iden=self.iden)
-
+            variables=self.variables,
+            iden=self.iden
+            )
+    def add_timevars(self): 
+        self.variables=self.model.add_timevars(self.iden)
+        out=[var for nm,var in self.variables.iteritems()]
+        self.variables['inputvar']=self.inputvar
+        self.variables['statusvar']=self.statusvar
+        return out
+        
+    def update_vars(self):
+        for k,v in self.variables.iteritems(): self.variables[k]=value(v)
     def __str__(self): return 'bid {i}'.format(i=self.iden)
 
+
+def makeModel(polyText,multiplier=1, **kwargs):
+    polyCurve=multiplier * parsePolynomial(polyText)
+    if isLinear(polyCurve):
+        return LinearModel(polyText,multiplier,**kwargs)
+    elif isConvex(polyCurve):
+        return convexPWLmodel(polyText,multiplier,**kwargs)
+    else:
+        return PWLmodel(polyText,multiplier,**kwargs)
+    
+        
+    
 class PWLmodel(object):
     """
     Describes a piecewise linear model of a polynomial curve.
@@ -61,18 +81,17 @@ class PWLmodel(object):
         polyText='2+10P+0.1P^2',multiplier=1,
         minInput=0,maxInput=10000,
         numBreakpoints=default_num_breakpoints,
-        name='',iden='',inputNm='x',outputNm='y'):
+        inputNm='x',outputNm='y'):
                 
         vars(self).update(locals()) #set the input vars above to be part of class
         self.polyCurve=multiplier * parsePolynomial(polyText) #parse curve
-        def isLinear(P): return True if P.order<=1 else False #check if a linear polynomial
         if isLinear(self.polyCurve): self.numBreakpoints=2 #linear models only need 2 breakpoints
         inDiscrete=linspace(self.minInput, self.maxInput, 1e6) #fine discretization of the curve
         outDiscrete=polyval(self.polyCurve,inDiscrete)
-        self.bpInputs = linspace(self.minInput, self.maxInput, self.numBreakpoints) #interpolation to get pwl breakpoints
-        self.bpOutputs= interp(self.bpInputs,inDiscrete,outDiscrete)
+        self.bpInputs = [float(bpi) for bpi in linspace(self.minInput, self.maxInput, self.numBreakpoints)] #interpolation to get pwl breakpoints
+        self.bpOutputs= [float(bpo) for bpo in interp(self.bpInputs,inDiscrete,outDiscrete)]
         self.segments=range(1,len(self.bpInputs))
-
+        
     def plot(self,P=None,filename=None,showPW=True):
         inDiscrete=linspace(self.minInput, self.maxInput, 1e6)
         outDiscrete=polyval(self.polyCurve,inDiscrete)
@@ -89,38 +108,48 @@ class PWLmodel(object):
         outDiscrete=polyval(deriv,inDiscrete)
         linePlotted, =plot(inDiscrete,outDiscrete,linestyle=linestyle)
         if P is not None: plot(P,polyval(deriv,P), 'o', c=linePlotted.get_color(), markersize=8, linewidth=2, alpha=0.7)
-        return linePlotted
-                
-    def addTimeVars(self,iden):
-        S=[] #S: segment of cost curve is active
-        F=[] #F: breakpoint weighting fraction
-        for segNum,seg in enumerate(self.segments):  S.append(newVar(kind='Binary', name='{iden}_s{segNum}'.format(segNum=segNum,iden=iden))) 
-        for bpNum,bp in enumerate(self.bpInputs):    F.append(newVar(low=0,high=1,  name='{iden}_f{bpNum}'.format( bpNum=bpNum,  iden=iden)))
-        return S,F
-    def constraints(self,S,F,inputVar,status,iden):
+        return linePlotted            
+    def add_timevars(self,iden):
+        variables={}
+        #S: segment of cost curve is active
+        #F: breakpoint weighting fraction
+        for segNum,seg in enumerate(self.segments):  
+            name='{iden}_s{segNum}'.format(segNum=segNum,iden=iden)
+            variables[name] = newVar(kind='Binary', name=name)
+         
+        for bpNum,bp in enumerate(self.bpInputs):
+            name='{iden}_f{bpNum}'.format( bpNum=bpNum,  iden=iden)
+            variables[name] = newVar(low=0,high=1,  name=name)
+        return variables 
+    def constraints(self,variables,iden):
         """
         Create the constraints for a single time instance of 
           a piecewise linear model. 
           
-        :param S: list of segment status variables
-        :param F: list of breakpoint fraction variables
-        :param inputVar: variable specifying the bidder power
-        :param status: variable specifying whether bidder is committed
+        :param variables: a dictionary of the variables for the bid
         :param iden: identifying string for the bidder and time interval
         
         :returns: a dictionary of constraints
         """
         constraints=dict()
+        status = variables['statusvar']
+        if status in (True,False): status=1 if status else 0 #convert bool to integer for coopr 
+        inputVar = variables['inputvar']
+        S = [variables['{iden}_s{segNum}'.format(segNum=s,iden=iden)] for s in range(len(self.segments))] 
+        F = [variables['{iden}_f{bpNum}'.format(bpNum=f,iden=iden)] for f in range(len(self.bpInputs))]
+
         constraints['oneActiveSegment '+iden]= ( sumVars(S)== status )
-        constraints['fractionSums '+iden] =    ( sumVars(F) == status )
-        constraints['computeInput '+iden] =    ( inputVar == sumVars( elementwiseMultiply(F,self.bpInputs) ) )
+        constraints['fractionSums '+iden]    = ( sumVars(F) == status )
+        constraints['computeInput '+iden]    = ( inputVar == sumVars( elementwiseMultiply(F,self.bpInputs) ) )
         constraints['firstSegment '+iden]    = ( F[0]<=S[0] )
         constraints['lastSegment '+iden]     = ( F[-1]<=S[-1] )
         for b in range(1,self.numBreakpoints-1): 
             name='midSegment {iden} b{bnum}'.format(iden=iden,bnum=b)
-            constraints[name] =                ( F[b] <= sumVars([S[b-1],S[b]]) )
+            constraints[name]                = ( F[b] <= sumVars([S[b-1],S[b]]) )
         return constraints
-    def output(self,F): return sumVars( elementwiseMultiply(F,self.bpOutputs) )
+    def output(self,variables,iden): 
+        F = [variables['{iden}_f{bpNum}'.format(bpNum=f,iden=iden)] for f in range(len(self.bpInputs))]
+        return sumVars( elementwiseMultiply(F,self.bpOutputs) )
     def trueOutput(self,input): return polyval( self.polyCurve,         value(input) )
     def incOutput(self,input):  return polyval( polyder(self.polyCurve),value(input) )
     def texrepresentation(self,digits=3):
@@ -142,6 +171,84 @@ class PWLmodel(object):
         if texstr[0]=='+': texstr=texstr[1:]
         return texstr
 
+
+class convexPWLmodel(PWLmodel):
+    def __init__(self,
+        polyText='2+10P+0.1P^2',multiplier=1,
+        minInput=0,maxInput=10000,
+        numBreakpoints=default_num_breakpoints,
+        inputNm='x',outputNm='y'):
+        
+        def linear_equation(x,m,b): return m*x+b
+        def make_lineareq(x1,y1,x2,y2):
+            m=(y2-y1)/(x2-x1)
+            b=y1-x1*m
+            return lambda x: linear_equation(x,m,b)
+        
+        vars(self).update(locals()) #set the input vars above to be part of class
+        self.polyCurve=multiplier * parsePolynomial(polyText) #parse curve
+        inDiscrete=linspace(self.minInput, self.maxInput, 1e6) #fine discretization of the curve
+        outDiscrete=polyval(self.polyCurve,inDiscrete)
+        self.bpInputs = [float(bpi) for bpi in linspace(self.minInput, self.maxInput, self.numBreakpoints)] #interpolation to get pwl breakpoints
+        self.bpOutputs= [float(bpo) for bpo in interp(self.bpInputs,inDiscrete,outDiscrete)]
+        self.segment_lines=[]
+        for b,x1 in enumerate(self.bpInputs[:-1]):
+            x2,y2=self.bpInputs[b+1],self.bpOutputs[b+1]
+            y1=self.bpOutputs[b]
+            self.segment_lines.append(make_lineareq(x1,y1,x2,y2))
+        
+    def add_timevars(self,iden):
+        variables={}
+        name = 'bidCost_'+iden
+        variables[name] = newVar(name=name,high=float(max(self.bpOutputs)))
+        return variables
+    def constraints(self,variables,iden):
+        constraints=dict()
+        for b,line in enumerate(self.segment_lines): 
+            nm='cost_linearized_{}_b{}'.format(iden,b)
+            constraints[nm]= variables['bidCost_'+iden] >= line(variables['inputvar'])
+        return constraints
+    def output(self,variables,iden): return variables['bidCost_'+iden]
+    def trueOutput(self,input): return polyval( self.polyCurve,         value(input) )
+    def incOutput(self,input):  return polyval( polyder(self.polyCurve),value(input) )
+    def plot(self,P=None,filename=None,showPW=True):
+        inDiscrete=linspace(self.minInput, self.maxInput, 1e6)
+        outDiscrete=polyval(self.polyCurve,inDiscrete)
+        if showPW: 
+            x=linspace(self.minInput, self.maxInput, 1e6)
+            for line in self.segment_lines: plot(x,line(x),'k--') #show piecewise linearization
+        plot(inDiscrete,outDiscrete,'-')                      #show continuous curve
+        xlabel(self.inputNm)
+        ylabel(self.outputNm)
+        if P is not None: plot(P,polyval(self.polyCurve,P), 'o', c=linePlotted.get_color(), markersize=8, linewidth=2, alpha=0.7) 
+        if filename is not None: savefig(filename)
+        else: show()
+        return
+
+class LinearModel(PWLmodel):
+    def __init__(self,
+        polyText='2+10P+0.1P^2',multiplier=1,
+        minInput=0,maxInput=10000,
+        numBreakpoints=None,
+        inputNm='x',outputNm='y'):
+        
+        self.polyCurve = multiplier * parsePolynomial(polyText)
+        self.minInput=minInput
+        self.maxInput=maxInput
+        
+    def add_timevars(self,iden): return dict()
+    def constraints(self,variables,iden): return dict()
+    def output(self,variables,iden): return polyval( self.polyCurve,variables['inputvar'] )    
+    
+    
+def isLinear(P):
+    """Check if a polynomial object is linear using its order attribute."""
+    return True if P.order<=1 else False 
+def isConvex(P):
+    #convexity is complicated: http://plus.maths.org/content/convexity
+    #so for now: if all polynomial coefs are positive, call it convex
+    return True if all(coef>=0 for coef in P.c) else False
+    
 def parsePolynomial(s):
     """
     Parse a string into a numpy polynomial object.
