@@ -3,6 +3,7 @@ Provide the defaults and data for other modules.
 Specifically, the :class:`~powersystems.Generator` defaults
 are provided by :data:`generator_defaults`.
 """
+import logging
 
 generator_kinds=[
     'generic',
@@ -32,7 +33,7 @@ generator_defaults=dict(
         baseload=2,coal=2,nuclear=18
         ),
     rampratemax=dict(
-        generic=1000,
+        generic=None,
         baseload=200,   coal=200,nuclear=100,
         shoulder=500, naturalgas=500, steamturbine=500,
         peaker=3000, combustionturbine=3000,
@@ -53,19 +54,24 @@ generator_defaults=dict(
         ),
     startupcost=dict(
         #from Henry Louie thesis
-        generic=          80000,
+        generic=              0, #if kind not specified, no startup cost 
         coal=            107800,
         nuclear=         283500,
         steamturbine=     73500,
         combustionturbine=36750,
         wind=                 0
         ),
-    isControllable=True
+    isControllable=True,
+    power=None
     )
 
 
-default_num_breakpoints=25
+cost_loadshedding = 100000000 #$/MWh
 
-optimization_package='pulp'
+default_num_breakpoints=5
 
-optimization_solver='glpk'
+optimization_package='coopr' #'pulp'
+
+optimization_solver='gurobi'
+available_solvers = ['glpk','gurobi']#,'cplex']
+logging_level= logging.INFO
