@@ -7,6 +7,7 @@ from the information found in the data.
 
 import powersystems
 import schedule
+from addons import * 
 from commonscripts import readCSV,csvColumn,flatten,unique,drop_case_spaces,getattrL, joindir
 
 import os,sys,logging
@@ -17,6 +18,7 @@ fields_lines={'name':'name','to':'To','from':'From','pmax':'Pmax'}
 fields_gens={
     'name':'name','type':'kind','kind':'kind','bus':'bus',
     'pmin':'Pmin','pmax':'Pmax',
+    'p':'power','pg':'power','power':'power', #for a non-controllable gen in an ED
     'rampratemin':'rampratemin','rampratemax':'rampratemax',
     'minuptime':'minuptime','mindowntime':'mindowntime',
     'costcurveequation':'costcurvestring', 
@@ -27,8 +29,14 @@ fields_loads={'name':'name','bus':'bus','type':'kind','kind':'kind',
             'p':'P','pd':'P', 'pmin':'Pmin','pmax':'Pmax',
             'schedulefilename':'schedulefilename',
             'bidequation':'costcurvestring','costcurveequation':'costcurvestring'}
-
-def parsedir(datadir='./tests/uc/',
+fields_initial={
+    'name':'name','generatorname':'name',
+    'status':'u','u':'u',
+    'p':'P','pg':'P','power':'P',
+    'hoursinstatus':'hoursinstatus',
+    'ic':None}
+    
+def parsedir(datadir='.',
         file_gens='generators.csv',
         file_loads='loads.csv',
         file_lines='lines.csv',
@@ -54,28 +62,27 @@ def parsedir(datadir='./tests/uc/',
     times=setup_times(file_gens,file_loads,datadir)
     
     #add loads
-    loads=build_class_list(file_loads,classname=powersystems.makeLoad,field_attr_map=fields_loads,times=times)
+    loads=build_class_list(file_loads,model=powersystems.makeLoad,field_attr_map=fields_loads,times=times)
     
     #add gens
-    generators=build_class_list(file_gens,classname=powersystems.makeGenerator,field_attr_map=fields_gens,times=times)
+    generators=build_class_list(file_gens,model=powersystems.makeGenerator,field_attr_map=fields_gens,times=times)
     
     #add initial conditions
     generators=setup_initialcond(file_init,generators,times)
     
     #add lines
-    try: lines=build_class_list(file_lines,classname=powersystems.Line,field_attr_map=fields_lines)
+    try: lines=build_class_list(file_lines,model=powersystems.Line,field_attr_map=fields_lines)
     except IOError: lines=[]
     
     #create buses list    
-    buses=setup_buses(loads,generators)
+    buses=powersystems.make_buses_list(loads,generators)
     
     return buses,lines,times
 
 def setup_initialcond(filename,generators,times):
     '''read initial conditions from spreadsheet and add information to each :class:`powersystems.Generator`.'''
     if len(times)<=1: return generators #for UC,ED no need to set initial status
-    field_attr_map={'name':'name','status':'u','p':'P','hoursinstatus':'hoursinstatus'}
-    validFields=field_attr_map.keys()
+    validFields=fields_initial.keys()
     
     initialTime = times.initialTime
     
@@ -86,26 +93,32 @@ def setup_initialcond(filename,generators,times):
         for gen in generators: gen.setInitialCondition(time=initialTime)
         return generators
         
-    try: attributes=[field_attr_map[drop_case_spaces(f)] for f in fields]
+    try: attributes=[fields_initial[drop_case_spaces(f)] for f in fields]
     except KeyError:
         print 'Field "{f}" is not in valid fields (case insensitive): {V}.'.format(f=f,V=validFields)
         raise
     
     
-    #add info to generators
+    #set initial condition for all gens to off
+    for g in generators: g.setInitialCondition(initialTime, u=False, P=0)
+
+
+    #overwrite initial condition for generators which are specified in the initial file
     genNames=[g.name for g in generators]
     nameCol = attributes.index('name')
+    excludeCols = [nameCol]
+    if None in attributes: excludeCols.append( attributes.index(None) )
     for row in data:
         inputs=dict()
         for c,elem in enumerate(row): 
-            if c==nameCol: continue
-            elif row[c] is not None: inputs[attributes[c]]=row[c]
+            if c in excludeCols: continue
+            elif elem is not None: inputs[attributes[c]]=elem
         g=genNames.index(row[nameCol])
         generators[g].setInitialCondition(time=initialTime,**inputs)
         
     return generators
 
-def build_class_list(filename,classname,field_attr_map,times=None):
+def build_class_list(filename,model,field_attr_map,times=None):
     """
     Create list of class instances from data in a spreadsheet.
     
@@ -117,27 +130,41 @@ def build_class_list(filename,classname,field_attr_map,times=None):
     
     :returns: a list of class objects
     """
-        
+    field_attr_map.update({'model':'model','modelschedule':'modelschedule'}) #add a model override
     dirname=os.path.dirname(filename)
     classL=[]
     validFields=field_attr_map.keys()
     data,fields=readCSV(filename,validFields)
     try: attributes=[field_attr_map[drop_case_spaces(f)] for f in fields]
     except KeyError:
-        raise KeyError('Field "{f}" is not in list'.format(f=f)+
-            'of valid fields (case insensitive): {V}.'.format(V=validFields))
+        msg='Field "{f}" is not in list of valid fields (case insensitive): {V}.'.format(f=f,V=validFields)
+        raise KeyError(msg)
+    
+    def getmodel(default,name,inputs):
+        model=default
+        newmodel=inputs.pop(name,None)
+        if newmodel is None:
+            return model
+        else:
+            modname,classname=newmodel.split('.')
+            return getattr(globals()[modname],classname)
+    
     
     index=0
     for row in data:
         inputs=dict()
         for c,elem in enumerate(row): 
             if row[c] is None: continue
-            elif attributes[c]=='schedulefilename':
-                schedulefilename=joindir(dirname,row[c])
-                inputs['schedule']=schedule.Schedule(schedulefilename,times)
             else: inputs[attributes[c]]= row[c]
         else:
-            classL.append( classname(index=index, **inputs) )
+            model_local         =getmodel(model,'model',inputs)
+            model_schedule_local=getmodel(schedule.makeSchedule,'modelschedule',inputs)
+
+            schedulefilename=inputs.pop('schedulefilename',None)
+            if schedulefilename is not None:
+                inputs['schedule']=model_schedule_local(joindir(dirname,schedulefilename),times)
+            
+            classL.append( model_local(index=index, **inputs) )
             index+=1
     return classL
 
@@ -154,14 +181,6 @@ def setup_times(file_gens,file_loads,datadir):
         :returns: a :class:`~schedule.Timelist` object
     """
 
-    def makeTimes(datetimeL):
-        '''convert list of datetime objects to Timelist() class'''
-        S=datetimeL[0]
-        I=datetimeL[1] - S #interval
-        E=datetimeL[-1] + I #one past the last time
-        times=schedule.Timelist(Start=S,End=E,interval=I)
-        times.setInitial()
-        return times
 
     def getTimeCol(filename):
         if filename is None: return []
@@ -180,7 +199,7 @@ def setup_times(file_gens,file_loads,datadir):
     
     if len(genScheds)==0 and len(loadScheds)==0:
         #this is a ED or OPF problem - only one time
-        times=schedule.Timelist([schedule.Time(Start='0:00',index=0)])
+        times=schedule.just_one_time()
         return times
 
 
@@ -193,44 +212,21 @@ def setup_times(file_gens,file_loads,datadir):
     timedateL=schedule.parse_timestrings(timestrL)
     
     if len(timestrL) == max(n for n in nT_loads+nT_gens): 
-        times=makeTimes(timedateL) #just one schedule
+        times=schedule.makeTimes(timedateL) #just one schedule
     else: 
         uniqueTimes=unique(timedateL)
         #make times from unique list
-        times=makeTimes(uniqueTimes)
-    
+        times=schedule.makeTimes(uniqueTimes)
+        
     #check for missing data problems by checking length
     nT=len(times)
-    if any(n!=nT if n else False for n in nT_loads ): raise ValueError('a load has schedule with inconsistent times. load schedule lengths={L} and there are {t} times.'.format(L=nT_loads,t=nT))
-    if any(n!=nT if n else False for n in nT_gens  ): raise ValueError('a generator has schedule with inconsistent times. gen schedule lengths={L} and there are {t} times.'.format(L=nT_gens,t=nT))
+    if any(n!=nT if n else False for n in nT_loads): 
+        msg='a load has schedule with inconsistent times. load schedule lengths={L} and there are {t} times.'.format(L=nT_loads,t=nT)
+        raise ValueError(msg)
+    if any(n!=nT if n else False for n in nT_gens): 
+        msg='a generator has schedule with inconsistent times. gen schedule lengths={L} and there are {t} times.'.format(L=nT_gens,t=nT)
+        raise ValueError(msg)
     return times
-    
-    
-def setup_buses(loads,generators):
-    """Create list of :class:`powersystems.Bus` objects 
-        from the load and generator bus names. Otherwise
-        (as in ED,UC) create just one (system)
-        :class:`powersystems.Bus` instance.
-        
-        :param loads: a list of :class:`powersystems.Load` objects
-        :param generators: a list of :class:`powersystems.Generator` objects
-        :returns: a list of :class:`powersystems.Bus` objects
-    """
-    busNameL=[]
-    busNameL.extend(getattrL(generators,'bus'))
-    busNameL.extend(getattrL(loads,'bus'))
-    busNameL=unique(busNameL)
-    buses=[]
-    swingHasBeenSet=False
-    for b,busNm in enumerate(busNameL):
-        newBus=powersystems.Bus(name=busNm,index=b)
-        for gen in generators: 
-            if gen.bus==newBus.name: newBus.generators.append(gen) 
-            if not swingHasBeenSet: newBus.isSwing=swingHasBeenSet=True
-        for ld in loads: 
-            if ld.bus==newBus.name: newBus.loads.append(ld)             
-        buses.append(newBus)
-    return buses
 
 if __name__ == "__main__": 
     if len(sys.argv)==1: parsedir()
