@@ -3,6 +3,7 @@ Time and schedule related models.
 """
 
 import dateutil
+import logging
 from commonscripts import hours,parseTime, readCSV,getclass_inlist, drop_case_spaces,transpose,frange,getattrL,getTimeFormat
 
 
@@ -40,10 +41,31 @@ class Time(object):
         rangeLenHrs = hours(self.End-self.Start)
         return [self.Start + hours(t) for t in frange(0,rangeLenHrs,intervalStepHrs)]
     def __sub__(self, other): return self.End-other.Start
+    def __add__(self, other): return self.Start+other
     def __str__(self): 
         try: return 't{ind:02d}'.format(ind=self.index)    
         except ValueError: return 't_{ind}'.format(ind=self.index) #index is str    
+    def __unicode__(self):
+        return unicode(self.Start)+' to '+unicode(self.End)
 
+def makeTimes(datetimeL):
+    '''convert list of datetime objects to Timelist() class'''
+    S=datetimeL[0]
+    I=datetimeL[1] - S #interval
+    E=datetimeL[-1] + I #one past the last time
+    times=Timelist(Start=S,End=E,interval=I)
+    times.setInitial()
+    return times
+
+def make_times_basic(N):
+    ''' make a :class:`schedule.Timelist` of N times, assume hourly interval.'''
+    S=dateutil.parser.parse('0:00')
+    I=dateutil.parser.parse('1:00')-S
+    E=S+I*(N)    
+    times=Timelist(Start=S,End=E,interval=I)
+    times.setInitial()
+    return times
+    
 class Timelist(object):
     """
     A container for :class:`schedule.Time` objects.
@@ -91,6 +113,7 @@ class Timelist(object):
         else: self.initialTime = Time(Start=self.Start-self.interval, interval=self.interval)
         self.wInitial = tuple([self.initialTime] + list(self.times))
     def subdivide(self,hrsperdivision=24,hrsinterval=None):
+        ### add offset=0 to this
         """
         Subdivide a list of times into serval lists,  each list
         spanning `hrsperdivision` with intervals of `hrsinterval`.
@@ -139,7 +162,7 @@ class Timelist(object):
         return newtimesL
         
         
-def setUpSchedule(filename,times):
+def makeSchedule(filename,times):
     """
     Read time and power information from spreadsheet file.
     """
@@ -151,28 +174,30 @@ def setUpSchedule(filename,times):
     data,fields=readCSV(filename,validFields)
     attributes=[mapFieldsToAttributes[drop_case_spaces(f)] for f in fields]
     data_power=transpose(data)[attributes.index('P')] 
-    return dict(zip(times,data_power))
+    return Schedule(times,data_power)
     
 class Schedule(object):
     """
     Describes a schedule of times and corresponding power values.
-    In basic use is mostly a container for a dictionary keyed by
+    Mostly a container for a dictionary keyed by
     :py:class:`~schedule.Time` objects.
-    
-    If inputs are `P` and `times`: create dictionary directly.
-    
-    If inputs are `filename` and `times`: 
-    use :meth:`~schedule.setUpSchedule` to read information from 
-    spreadsheet file.
     """
-    def __init__(self,filename=None,times=None,P=None):
-        if P and times and not filename: self.P=dict(zip(times,P))
-        else: self.P=setUpSchedule(filename,times)
-        self.times=times
+    def __init__(self,times=None,P=None):
+        self.P=dict(zip(times,P))
         self.interval=times.interval
         self.intervalhrs = times.intervalhrs
         self.maxvalue=max(self.P.values())
-
+    def __imul__(self,multiplier):
+        """
+        Multiplies each power value in schedule by a multiplier.        
+        Usage: schedule*=.9 
+        would give a schedule with 90 percent of the power.
+        """
+        for t,p in self.P.iteritems():
+            self.P[t]=p*multiplier
+        return self
+    def __repr__(self):
+        return sorted([(str(t.Start),p) for t,p in self.P.iteritems()])    
     def getEnergy(self,timeperiod):
         """
         get the amount of energy in a time period 
@@ -192,19 +217,25 @@ class Schedule(object):
         >>> s.getEnergy(tLarger)
         310.0
         """
-        
+        times=self.P.keys()
         try: return self.P[timeperiod]
         except KeyError:
             if timeperiod.interval>self.interval: 
                 #energy of time is sum all energy in times within
                 tstarts=timeperiod.Range(self.interval)
-                period_Times=getclass_inlist(self.times,tstarts,attribute='Start')
-                return sum([self.getEnergy(t)*self.intervalhrs for t in period_Times])
+                period_Times=getclass_inlist(times,tstarts,attribute='Start')
+                return sum([self.getEnergy(t) for t in period_Times]) *self.intervalhrs
             elif timeperiod.interval==self.interval:
-                t=getattrL(self.times,'Start').index(timeperiod.Start)
-                return self.getEnergy(self.times[t])
+                t=getattrL(times,'Start').index(timeperiod.Start)
+                return self.getEnergy(times[t])
             else: raise
-
+class FixedSchedule(Schedule):
+    def __init__(self,times=None,P=None): self.P=P
+    def getEnergy(self,timeperiod=None): return self.P
+    
+def just_one_time():
+    """For a single-time problem, generate a Timelist with just one time in it."""
+    return Timelist([Time(Start='0:00',index=0)])
 def parse_timestrings(timestringsL):
     """
     Convert list of strings to list of :py:class:`datetime.datetime`
